# Copyright © 2021 Arrikto Inc.  All Rights Reserved.

"""Kale SDK.

This script trains an ML pipeline to solve a binary classification task.
"""

from kale.sdk import has_metrics, pipeline, step
from kale.sdk.logging import log_metric
from sklearn.datasets import make_classification
from sklearn.linear_model import LogisticRegression
from sklearn.metrics import accuracy_score
from sklearn.model_selection import train_test_split
from kubeflow import katib


# The Katib experiment definition.
katib_experiment = katib.V1beta1ExperimentSpec(
    max_trial_count=3,
    parallel_trial_count=1,
    max_failed_trial_count=0,
    algorithm=katib.V1beta1AlgorithmSpec(
        algorithm_name="grid"
    ),
    objective=katib.V1beta1ObjectiveSpec(
        type="maximize",
        objective_metric_name="accuracy"
    ),
    parameters=[katib.V1beta1ParameterSpec(
        name="c",
        parameter_type="double",
        feasible_space=katib.V1beta1FeasibleSpace(
            min="0.1",
            max="1.0",
            step="0.3"),
    ), katib.V1beta1ParameterSpec(
        name="penalty",
        parameter_type="categorical",
        feasible_space=katib.V1beta1FeasibleSpace(
            list=["l2", "none"])
    )],
)


@step(name="data_loading")
def load(random_state):
    """Create a random dataset for binary classification."""
    rs = int(random_state)
    x, y = make_classification(random_state=rs)
    return x, y


@step(name="data_split")
def split(x, y):
    """Split the data into train and test sets."""
    x, x_test, y, y_test = train_test_split(x, y, test_size=0.1)
    return x, x_test, y, y_test


@step(name="model_training")
def train(x, x_test, y, training_iterations):
    """Train a Logistic Regression model."""
    iters = int(training_iterations)
    model = LogisticRegression(max_iter=iters)
    model.fit(x, y)
    return model


@has_metrics
@step(name="model_evaluation")
def evaluate(model, x_test, y_test):
    """Evaluate the model on the test dataset."""
    y_pred = model.predict(x_test)
    accuracy = accuracy_score(y_test, y_pred)
    log_metric(name="accuracy", value=accuracy)


@pipeline(name="binary-classification", experiment="kale-tutorial")
def ml_pipeline(rs=42, iters=100):
    """Run the ML pipeline."""
    x, y = load(rs)
    x, x_test, y, y_test = split(x, y)
    model = train(x, x_test, y, iters)
    evaluate(model, x_test, y_test)


if __name__ == "__main__":
    ml_pipeline(rs=42, iters=100)
